/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002-2006  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <fstream>
#include <string>
#include <glib.h>
#include <pan/general/debug.h>
#include <pan/general/foreach.h>
#include "ui-prefs.h"

using namespace pan;

namespace
{
  struct MyContext {
    UI_Prefs& prefs;
    std::string text;
    MyContext (UI_Prefs& p): prefs(p) {}
  };

  // called for open tags <foo bar='baz'>
  void start_element (GMarkupParseContext *context,
                      const gchar         *element_name,
                      const gchar        **attribute_names,
                      const gchar        **attribute_vals,
                      gpointer             user_data,
                      GError             **error)
  {
    MyContext& mc (*static_cast<MyContext*>(user_data));
    mc.text.clear ();
  }

  // Called for close tags </foo>
  void end_element    (GMarkupParseContext *context,
                       const gchar         *element_name,
                       gpointer             user_data,
                       GError             **error)
  {
    MyContext& mc (*static_cast<MyContext*>(user_data));

    if (!strcmp(element_name, "single-click-activates-group"))
      mc.prefs.single_click_activates_group = mc.text.empty() || (tolower(mc.text[0])!='f');
    else if (!strcmp(element_name, "single-click-activates-article"))
      mc.prefs.single_click_activates_article = mc.text.empty() || (tolower(mc.text[0])!='f');
  }

  void text (GMarkupParseContext *context,
             const gchar         *text,
             gsize                text_len,  
             gpointer             user_data,
             GError             **error)
  {
    static_cast<MyContext*>(user_data)->text.assign (text, text_len);
  }
}

void
UI_Prefs :: set_from_xml (const StringView& xml)
{
  defaults ();

  MyContext mc (*this);
  GMarkupParser p;
  p.start_element = start_element;
  p.end_element = end_element;
  p.text = text;
  p.passthrough = 0;
  p.error = 0;
  GMarkupParseContext* c (
    g_markup_parse_context_new (&p, (GMarkupParseFlags)0, &mc, 0));
  GError * gerr (0);
  if (g_markup_parse_context_parse (c, xml.str, xml.len, &gerr)) {
    // FIXME
  }
  g_markup_parse_context_free (c);
}

void
UI_Prefs :: set_from_file (const StringView& filename)
{
  gchar * txt (0);
  gsize len (0);
  if (g_file_get_contents (filename.to_string().c_str(), &txt, &len, 0))
    set_from_xml (StringView(txt,len));
  g_free (txt);
}

namespace
{
  const int indent_char_len (2);

  std::string indent (int depth) { return std::string(depth*indent_char_len, ' '); }

  std::string escaped (const std::string& s)
  {
    char * pch = g_markup_escape_text (s.c_str(), s.size());
    const std::string ret (pch);
    g_free (pch);
    return ret;
  }

  std::string escaped (const bool b)
  {
    return b ? "true" : "false";
  }
}

void
UI_Prefs :: serialize (std::ostream&  out) const
{
  int depth (0);
  out << "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n";
  out << indent(depth++) << "<preferences>\n";
  out << indent(depth) << "<single-click-activates-group>" << escaped(single_click_activates_group) << "</single-click-activates-group>\n";
  out << indent(depth) << "<single-click-activates-article>" << escaped(single_click_activates_article) << "</single-click-activates-article>\n";
  out << indent(--depth) << "</preferences>\n";
}

void
UI_Prefs :: defaults ()
{
  single_click_activates_group = true;
  single_click_activates_article = true;
}

UI_Prefs :: UI_Prefs (const StringView& filename):
  _filename (filename.to_string())
{
  defaults ();
  set_from_file (_filename);
}

UI_Prefs :: ~UI_Prefs ()
{
  save ();
}

void 
UI_Prefs :: save ()
{
  std::ofstream out (_filename.c_str());
  serialize (out);
  out.close ();
}
